@@ -12,17 +12,17 @@
 %                          MagickCore Cipher Methods                          %
 %                                                                             %
 %                             Software Design                                 %
-%                               John Cristy                                   %
+%                                  Cristy                                     %
 %                               March  2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -78,11 +78,11 @@ typedef struct _AESInfo
     *encipher_key,
     *decipher_key;
 
-  long
+  ssize_t
     rounds,
     timestamp;
 
-  unsigned long
+  size_t
     signature;
 } AESInfo;
 
@@ -193,7 +193,7 @@ static AESInfo *AcquireAESInfo(void)
   aes_info=(AESInfo *) AcquireMagickMemory(sizeof(*aes_info));
   if (aes_info == (AESInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  (void) ResetMagickMemory(aes_info,0,sizeof(*aes_info));
+  (void) memset(aes_info,0,sizeof(*aes_info));
   aes_info->blocksize=AESBlocksize;
   aes_info->key=AcquireStringInfo(32);
   aes_info->encipher_key=(unsigned int *) AcquireQuantumMemory(60UL,sizeof(
@@ -204,8 +204,8 @@ static AESInfo *AcquireAESInfo(void)
       (aes_info->encipher_key == (unsigned int *) NULL) ||
       (aes_info->decipher_key == (unsigned int *) NULL))
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  aes_info->timestamp=(long) time(0);
-  aes_info->signature=MagickSignature;
+  aes_info->timestamp=(ssize_t) time(0);
+  aes_info->signature=MagickCoreSignature;
   return(aes_info);
 }
 
@@ -235,7 +235,7 @@ static AESInfo *DestroyAESInfo(AESInfo *aes_info)
 {
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(aes_info != (AESInfo *) NULL);
-  assert(aes_info->signature == MagickSignature);
+  assert(aes_info->signature == MagickCoreSignature);
   if (aes_info->decipher_key != (unsigned int *) NULL)
     aes_info->decipher_key=(unsigned int *) RelinquishMagickMemory(
       aes_info->decipher_key);
@@ -244,7 +244,7 @@ static AESInfo *DestroyAESInfo(AESInfo *aes_info)
       aes_info->encipher_key);
   if (aes_info->key != (StringInfo *) NULL)
     aes_info->key=DestroyStringInfo(aes_info->key);
-  aes_info->signature=(~MagickSignature);
+  aes_info->signature=(~MagickCoreSignature);
   aes_info=(AESInfo *) RelinquishMagickMemory(aes_info);
   return(aes_info);
 }
@@ -281,7 +281,7 @@ static AESInfo *DestroyAESInfo(AESInfo *aes_info)
 static inline void AddRoundKey(const unsigned int *ciphertext,
   const unsigned int *key,unsigned int *plaintext)
 {
-  register long
+  register ssize_t
     i;
 
   /*
@@ -291,7 +291,7 @@ static inline void AddRoundKey(const unsigned int *ciphertext,
     plaintext[i]=key[i] ^ ciphertext[i];
 }
 
-static inline unsigned char ByteMultiply(const unsigned char alpha,
+static inline unsigned int ByteMultiply(const unsigned char alpha,
   const unsigned char beta)
 {
   /*
@@ -299,7 +299,7 @@ static inline unsigned char ByteMultiply(const unsigned char alpha,
   */
   if ((alpha == 0) || (beta == 0))
     return(0);
-  return(InverseLog[(Log[alpha]+Log[beta]) % 0xff]);
+  return((unsigned int) InverseLog[(Log[alpha]+Log[beta]) % 0xff]);
 }
 
 static inline unsigned int ByteSubTransform(unsigned int x,
@@ -311,8 +311,10 @@ static inline unsigned int ByteSubTransform(unsigned int x,
   /*
     Non-linear layer resists differential and linear cryptoanalysis attacks.
   */
-  key=(s_box[x & 0xff]) | (s_box[(x >> 8) & 0xff] << 8) |
-    (s_box[(x >> 16) & 0xff] << 16) | (s_box[(x >> 24) & 0xff] << 24);
+  key=((unsigned int) s_box[x & 0xff]) |
+    ((unsigned int) s_box[(x >> 8) & 0xff] << 8) |
+    ((unsigned int) s_box[(x >> 16) & 0xff] << 16) |
+    ((unsigned int) s_box[(x >> 24) & 0xff] << 24);
   return(key);
 }
 
@@ -337,7 +339,7 @@ static void FinalizeRoundKey(const unsigned int *ciphertext,
   {
     value=ciphertext[i] ^ key[i];
     for (j=0; j < 4; j++)
-      *p++=(value >> (8*j)) & 0xff;
+      *p++=(unsigned char) ((value >> (8*j)) & 0xff);
   }
   /*
     Reset registers.
@@ -380,7 +382,7 @@ static inline unsigned int RotateLeft(const unsigned int x)
 static void EncipherAESBlock(AESInfo *aes_info,const unsigned char *plaintext,
   unsigned char *ciphertext)
 {
-  register long
+  register ssize_t
     i,
     j;
 
@@ -483,8 +485,8 @@ static void EncipherAESBlock(AESInfo *aes_info,const unsigned char *plaintext,
     Reset registers.
   */
   alpha=0;
-  (void) ResetMagickMemory(key,0,sizeof(key));
-  (void) ResetMagickMemory(text,0,sizeof(text));
+  (void) memset(key,0,sizeof(key));
+  (void) memset(text,0,sizeof(text));
 }
 
 /*
@@ -519,11 +521,19 @@ static void EncipherAESBlock(AESInfo *aes_info,const unsigned char *plaintext,
 %
 */
 
-static inline size_t MagickMin(const size_t x,const size_t y)
+static inline void IncrementCipherNonce(const size_t length,
+  unsigned char *nonce)
 {
-  if (x < y)
-    return(x);
-  return(y);
+  register ssize_t
+    i;
+
+  for (i=(ssize_t) (length-1); i >= 0; i--)
+  {
+    nonce[i]++;
+    if (nonce[i] != 0)
+      return;
+  }
+  ThrowFatalException(ResourceLimitFatalError,"Sequence wrap error `%s'");
 }
 
 MagickExport MagickBooleanType DecipherImage(Image *image,
@@ -553,18 +563,18 @@ MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
   AESInfo
     *aes_info;
 
+  CacheView
+    *image_view;
+
   const unsigned char
     *digest;
 
-  IndexPacket
-    *indexes;
-
-  long
-    y;
-
   MagickBooleanType
     proceed;
 
+  MagickSizeType
+    extent;
+
   QuantumInfo
     *quantum_info;
 
@@ -580,6 +590,9 @@ MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
   size_t
     length;
 
+  ssize_t
+    y;
+
   StringInfo
     *key,
     *nonce;
@@ -589,30 +602,22 @@ MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
     output_block[AESBlocksize],
     *pixels;
 
-  CacheView
-    *image_view;
-
   /*
     Generate decipher key and nonce.
   */
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   if (passkey == (const StringInfo *) NULL)
     return(MagickTrue);
-  quantum_info=AcquireQuantumInfo((const ImageInfo *) NULL,image);
-  if (quantum_info == (QuantumInfo *) NULL)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      image->filename);
   aes_info=AcquireAESInfo();
   key=CloneStringInfo(passkey);
   if (key == (StringInfo *) NULL)
     {
       aes_info=DestroyAESInfo(aes_info);
-      quantum_info=DestroyQuantumInfo(quantum_info);
       ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
         image->filename);
     }
@@ -621,7 +626,6 @@ MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
     {
       key=DestroyStringInfo(key);
       aes_info=DestroyAESInfo(aes_info);
-      quantum_info=DestroyQuantumInfo(quantum_info);
       ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
         image->filename);
     }
@@ -629,52 +633,68 @@ MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
   key=DestroyStringInfo(key);
   signature_info=AcquireSignatureInfo();
   UpdateSignature(signature_info,nonce);
-  SetStringInfoLength(nonce,sizeof(quantum_info->extent));
-  SetStringInfoDatum(nonce,(const unsigned char *) &quantum_info->extent);
+  extent=(MagickSizeType) image->columns*image->rows;
+  SetStringInfoLength(nonce,sizeof(extent));
+  SetStringInfoDatum(nonce,(const unsigned char *) &extent);
   UpdateSignature(signature_info,nonce);
-  FinalizeSignature(signature_info);
-  (void) ResetMagickMemory(input_block,0,sizeof(input_block));
-  digest=GetStringInfoDatum(GetSignatureDigest(signature_info));
-  (void) CopyMagickMemory(input_block,digest,MagickMin(AESBlocksize,
-    GetSignatureDigestsize(signature_info))*sizeof(*input_block));
   nonce=DestroyStringInfo(nonce);
+  FinalizeSignature(signature_info);
+  (void) memset(input_block,0,sizeof(input_block));
+  digest=GetStringInfoDatum(GetSignatureDigest(signature_info));
+  (void) memcpy(input_block,digest,MagickMin(AESBlocksize,
+    GetSignatureDigestsize(signature_info))*sizeof(*input_block));
   signature_info=DestroySignatureInfo(signature_info);
   /*
     Convert cipher pixels to plain pixels.
   */
+  quantum_info=AcquireQuantumInfo((const ImageInfo *) NULL,image);
+  if (quantum_info == (QuantumInfo *) NULL)
+    {
+      aes_info=DestroyAESInfo(aes_info);
+      ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+        image->filename);
+    }
   quantum_type=GetQuantumType(image,exception);
   pixels=GetQuantumPixels(quantum_info);
-  image_view=AcquireCacheView(image);
-  for (y=0; y < (long) image->rows; y++)
+  image_view=AcquireAuthenticCacheView(image,exception);
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register long
+    register ssize_t
+      i,
       x;
 
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
-    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     length=ExportQuantumPixels(image,image_view,quantum_info,quantum_type,
       pixels,exception);
     p=pixels;
-    for (x=0; x < (long) length; x++)
+    for (x=0; x < (ssize_t) length; x+=AESBlocksize)
     {
-      (void) CopyMagickMemory(output_block,input_block,AESBlocksize*
+      (void) memmove(output_block,input_block,AESBlocksize*
         sizeof(*output_block));
+      IncrementCipherNonce(AESBlocksize,input_block);
       EncipherAESBlock(aes_info,output_block,output_block);
-      (void) CopyMagickMemory(input_block,input_block+1,(AESBlocksize-1)*
-        sizeof(*input_block));
-      input_block[AESBlocksize-1]=(*p);
-      *p++^=(*output_block);
+      for (i=0; i < AESBlocksize; i++)
+        p[i]^=output_block[i];
+      p+=AESBlocksize;
+    }
+    (void) memmove(output_block,input_block,AESBlocksize*sizeof(*output_block));
+    EncipherAESBlock(aes_info,output_block,output_block);
+    for (i=0; x < (ssize_t) length; x++)
+    {
+      p[i]^=output_block[i];
+      i++;
     }
     (void) ImportQuantumPixels(image,image_view,quantum_info,quantum_type,
       pixels,exception);
     if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       break;
-    proceed=SetImageProgress(image,DecipherImageTag,y,image->rows);
+    proceed=SetImageProgress(image,DecipherImageTag,(MagickOffsetType) y,
+      image->rows);
     if (proceed == MagickFalse)
       break;
   }
@@ -688,9 +708,9 @@ MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
   */
   quantum_info=DestroyQuantumInfo(quantum_info);
   aes_info=DestroyAESInfo(aes_info);
-  (void) ResetMagickMemory(input_block,0,sizeof(input_block));
-  (void) ResetMagickMemory(output_block,0,sizeof(output_block));
-  return(y == (long) image->rows ? MagickTrue : MagickFalse);
+  (void) memset(input_block,0,sizeof(input_block));
+  (void) memset(output_block,0,sizeof(output_block));
+  return(y == (ssize_t) image->rows ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -752,21 +772,21 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
   AESInfo
     *aes_info;
 
+  CacheView
+    *image_view;
+
   char
     *signature;
 
   const unsigned char
     *digest;
 
-  IndexPacket
-    *indexes;
-
-  long
-    y;
-
   MagickBooleanType
     proceed;
 
+  MagickSizeType
+    extent;
+
   QuantumInfo
     *quantum_info;
 
@@ -782,6 +802,9 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
   size_t
     length;
 
+  ssize_t
+    y;
+
   StringInfo
     *key,
     *nonce;
@@ -791,32 +814,24 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
     output_block[AESBlocksize],
     *pixels;
 
-  CacheView
-    *image_view;
-
   /*
     Generate encipher key and nonce.
   */
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   if (passkey == (const StringInfo *) NULL)
     return(MagickTrue);
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
     return(MagickFalse);
-  quantum_info=AcquireQuantumInfo((const ImageInfo *) NULL,image);
-  if (quantum_info == (QuantumInfo *) NULL)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      image->filename);
   aes_info=AcquireAESInfo();
   key=CloneStringInfo(passkey);
   if (key == (StringInfo *) NULL)
     {
       aes_info=DestroyAESInfo(aes_info);
-      quantum_info=DestroyQuantumInfo(quantum_info);
       ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
         image->filename);
     }
@@ -825,7 +840,6 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
     {
       key=DestroyStringInfo(key);
       aes_info=DestroyAESInfo(aes_info);
-      quantum_info=DestroyQuantumInfo(quantum_info);
       ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
         image->filename);
     }
@@ -833,57 +847,74 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
   key=DestroyStringInfo(key);
   signature_info=AcquireSignatureInfo();
   UpdateSignature(signature_info,nonce);
-  SetStringInfoLength(nonce,sizeof(quantum_info->extent));
-  SetStringInfoDatum(nonce,(const unsigned char *) &quantum_info->extent);
+  extent=(MagickSizeType) image->columns*image->rows;
+  SetStringInfoLength(nonce,sizeof(extent));
+  SetStringInfoDatum(nonce,(const unsigned char *) &extent);
   UpdateSignature(signature_info,nonce);
   nonce=DestroyStringInfo(nonce);
   FinalizeSignature(signature_info);
-  (void) ResetMagickMemory(input_block,0,sizeof(input_block));
+  (void) memset(input_block,0,sizeof(input_block));
   digest=GetStringInfoDatum(GetSignatureDigest(signature_info));
-  (void) CopyMagickMemory(input_block,digest,MagickMin(AESBlocksize,
+  (void) memcpy(input_block,digest,MagickMin(AESBlocksize,
     GetSignatureDigestsize(signature_info))*sizeof(*input_block));
   signature=StringInfoToHexString(GetSignatureDigest(signature_info));
   (void) SetImageProperty(image,"cipher:type","AES");
-  (void) SetImageProperty(image,"cipher:mode","CFB");
+  (void) SetImageProperty(image,"cipher:mode","CTR");
   (void) SetImageProperty(image,"cipher:nonce",signature);
   signature=DestroyString(signature);
   signature_info=DestroySignatureInfo(signature_info);
   /*
     Convert plain pixels to cipher pixels.
   */
+  quantum_info=AcquireQuantumInfo((const ImageInfo *) NULL,image);
+  if (quantum_info == (QuantumInfo *) NULL)
+    {
+      aes_info=DestroyAESInfo(aes_info);
+      ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+        image->filename);
+    }
   quantum_type=GetQuantumType(image,exception);
   pixels=GetQuantumPixels(quantum_info);
-  image_view=AcquireCacheView(image);
-  for (y=0; y < (long) image->rows; y++)
+  image_view=AcquireAuthenticCacheView(image,exception);
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register long
+    register ssize_t
+      i,
       x;
 
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
-    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     length=ExportQuantumPixels(image,image_view,quantum_info,quantum_type,
       pixels,exception);
     p=pixels;
-    for (x=0; x < (long) length; x++)
+    for (x=0; x < (ssize_t) length; x+=AESBlocksize)
     {
-      (void) CopyMagickMemory(output_block,input_block,AESBlocksize*
+      (void) memmove(output_block,input_block,AESBlocksize*
         sizeof(*output_block));
+      IncrementCipherNonce(AESBlocksize,input_block);
       EncipherAESBlock(aes_info,output_block,output_block);
-      *p^=(*output_block);
-      (void) CopyMagickMemory(input_block,input_block+1,(AESBlocksize-1)*
-        sizeof(*input_block));
-      input_block[AESBlocksize-1]=(*p++);
+      for (i=0; i < AESBlocksize; i++)
+        p[i]^=output_block[i];
+      p+=AESBlocksize;
+    }
+    (void) memmove(output_block,input_block,AESBlocksize*
+      sizeof(*output_block));
+    EncipherAESBlock(aes_info,output_block,output_block);
+    for (i=0; x < (ssize_t) length; x++)
+    {
+      p[i]^=output_block[i];
+      i++;
     }
     (void) ImportQuantumPixels(image,image_view,quantum_info,quantum_type,
       pixels,exception);
     if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       break;
-    proceed=SetImageProgress(image,EncipherImageTag,y,image->rows);
+    proceed=SetImageProgress(image,EncipherImageTag,(MagickOffsetType) y,
+      image->rows);
     if (proceed == MagickFalse)
       break;
   }
@@ -894,9 +925,9 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
   */
   quantum_info=DestroyQuantumInfo(quantum_info);
   aes_info=DestroyAESInfo(aes_info);
-  (void) ResetMagickMemory(input_block,0,sizeof(input_block));
-  (void) ResetMagickMemory(output_block,0,sizeof(output_block));
-  return(y == (long) image->rows ? MagickTrue : MagickFalse);
+  (void) memset(input_block,0,sizeof(input_block));
+  (void) memset(output_block,0,sizeof(output_block));
+  return(y == (ssize_t) image->rows ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -962,13 +993,13 @@ static inline unsigned int RotateRight(const unsigned int x)
 
 static void SetAESKey(AESInfo *aes_info,const StringInfo *key)
 {
-  long
+  register ssize_t
+    i;
+
+  ssize_t
     bytes,
     n;
 
-  register long
-    i;
-
   unsigned char
     *datum;
 
@@ -981,7 +1012,7 @@ static void SetAESKey(AESInfo *aes_info,const StringInfo *key)
   */
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(aes_info != (AESInfo *) NULL);
-  assert(aes_info->signature == MagickSignature);
+  assert(aes_info->signature == MagickCoreSignature);
   assert(key != (StringInfo *) NULL);
   n=4;
   aes_info->rounds=10;
@@ -1000,8 +1031,8 @@ static void SetAESKey(AESInfo *aes_info,const StringInfo *key)
     Generate crypt key.
   */
   datum=GetStringInfoDatum(aes_info->key);
-  (void) ResetMagickMemory(datum,0,GetStringInfoLength(aes_info->key));
-  (void) CopyMagickMemory(datum,GetStringInfoDatum(key),MagickMin(
+  (void) memset(datum,0,GetStringInfoLength(aes_info->key));
+  (void) memcpy(datum,GetStringInfoDatum(key),MagickMin(
     GetStringInfoLength(key),GetStringInfoLength(aes_info->key)));
   for (i=0; i < n; i++)
     aes_info->encipher_key[i]=datum[4*i] | (datum[4*i+1] << 8) |
@@ -1035,7 +1066,7 @@ static void SetAESKey(AESInfo *aes_info,const StringInfo *key)
     Reset registers.
   */
   datum=GetStringInfoDatum(aes_info->key);
-  (void) ResetMagickMemory(datum,0,GetStringInfoLength(aes_info->key));
+  (void) memset(datum,0,GetStringInfoLength(aes_info->key));
   alpha=0;
   beta=0;
 }
@@ -1077,11 +1108,11 @@ MagickExport MagickBooleanType DecipherImage(Image *image,
   const char *passphrase,ExceptionInfo *exception)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   (void) passphrase;
   ThrowBinaryException(ImageError,"CipherSupportNotEnabled",image->filename);
 }
@@ -1090,11 +1121,11 @@ MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
   const StringInfo *passkey,ExceptionInfo *exception)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   (void) passkey;
   ThrowBinaryException(ImageError,"CipherSupportNotEnabled",image->filename);
 }
@@ -1133,11 +1164,11 @@ MagickExport MagickBooleanType EncipherImage(Image *image,
   const char *passphrase,ExceptionInfo *exception)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   (void) passphrase;
   ThrowBinaryException(ImageError,"CipherSupportNotEnabled",image->filename);
 }
@@ -1146,11 +1177,11 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
   const StringInfo *passkey,ExceptionInfo *exception)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   (void) passkey;
   ThrowBinaryException(ImageError,"CipherSupportNotEnabled",image->filename);
 }
